@@ -8,19 +8,19 @@ install_scripts = '/usr/local/bin'
 config_dir = '/usr/local/share/linkchecker'
 install_data = '/usr/local'
 name = u'LinkChecker'
-version = u'7.9'
+version = u'8.3'
 author = u'Bastian Kleineidam'
-author_email = u'calvin@users.sourceforge.net'
+author_email = u'bastian.kleineidam@web.de'
 maintainer = u'Bastian Kleineidam'
-maintainer_email = u'calvin@users.sourceforge.net'
-url = u'http://linkchecker.sourceforge.net/'
+maintainer_email = u'bastian.kleineidam@web.de'
+url = u'http://wummel.github.com/linkchecker/'
 license = u'GPL'
 description = u'check links in web documents or full websites'
 long_description = u'Linkchecker features:\n\no recursive and multithreaded checking and site crawling\no output in colored or normal text, HTML, SQL, CSV, XML or a sitemap graph in different formats\no HTTP/1.1, HTTPS, FTP, mailto:, news:, nntp:, Telnet and local file links support\no restrict link checking with regular expression filters for URLs\no proxy support\no username/password authorization for HTTP, FTP and Telnet\no honors robots.txt exclusion protocol\no Cookie support\no HTML5 support\no HTML and CSS syntax check\no Antivirus check\no a command line, GUI and web interface\n\n'
 keywords = ['link', 'url', 'site', 'checking', 'crawling', 'verification', 'validation']
 platforms = ['UNKNOWN']
-fullname = u'LinkChecker-7.9'
+fullname = u'LinkChecker-8.3'
 contact = u'Bastian Kleineidam'
-contact_email = u'calvin@users.sourceforge.net'
-release_date = "10.6.2012"
+contact_email = u'bastian.kleineidam@web.de'
+release_date = "6.1.2013"
 portable = 0
