@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,10 +20,17 @@ Main function module for link checking.
 
 # imports and checks
 import sys
+# Needs Python >= 2.7 because we use dictionary based logging config
+# Needs Python >= 2.7.2 which fixed http://bugs.python.org/issue11467
 if not (hasattr(sys, 'version_info') or
-        sys.version_info < (2, 6, 0, 'final', 0)):
-    raise SystemExit("This program requires Python 2.6 or later.")
+        sys.version_info < (2, 7, 2, 'final', 0)):
+    raise SystemExit("This program requires Python 2.7.2 or later.")
 import os
+# add the custom linkcheck_dns directory to sys.path
+_dnspath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'linkcheck_dns'))
+if _dnspath not in sys.path:
+    sys.path.insert(0, _dnspath)
+del _dnspath
 import re
 
 from . import i18n
@@ -31,30 +38,31 @@ import _LinkChecker_configdata as configdata
 
 
 def main_is_frozen ():
+    """Return True iff running inside a py2exe-generated executable."""
     return hasattr(sys, "frozen")
 
 
 def module_path ():
+    """Return absolute directory of system executable."""
     return os.path.dirname(os.path.abspath(sys.executable))
 
 
 def get_install_data ():
+    """Return absolute path of LinkChecker data installation directory."""
     if main_is_frozen():
         return module_path()
     return configdata.install_data
 
 
 def get_config_dir ():
-    if main_is_frozen():
-        return os.path.join(module_path(), "share", "linkchecker")
-    return configdata.config_dir
+    """Return absolute path of LinkChecker example configuration."""
+    return os.path.join(get_install_data(), "share", "linkchecker")
 
 
 # application log areas
 LOG_ROOT = "linkcheck"
 LOG_CMDLINE = "linkcheck.cmdline"
 LOG_CHECK = "linkcheck.check"
-LOG_DNS = "linkcheck.dns"
 LOG_CACHE = "linkcheck.cache"
 LOG_GUI = "linkcheck.gui"
 LOG_THREAD = "linkcheck.thread"
@@ -63,12 +71,30 @@ lognames = {
     "checking": LOG_CHECK,
     "cache": LOG_CACHE,
     "gui": LOG_GUI,
-    "dns": LOG_DNS,
     "thread": LOG_THREAD,
     "all": LOG_ROOT,
 }
 lognamelist = ", ".join(repr(name) for name in lognames)
 
+# logging configuration
+configdict = {
+    'version': 1,
+    'loggers': {
+    },
+    'root': {
+      'level': 'DEBUG',
+    },
+}
+
+def init_log_config():
+    """Configure the application loggers."""
+    for applog in lognames.values():
+        # propagate except for root app logger 'linkcheck'
+        propagate = (applog != LOG_ROOT)
+        configdict['loggers'][applog] = dict(level='INFO', propagate=propagate)
+
+init_log_config()
+
 from . import log
 
 
@@ -80,13 +106,6 @@ class LinkCheckerInterrupt (StandardError):
     """Used for testing."""
     pass
 
-def add_intern_pattern (url_data, config):
-    """Add intern URL regex to config."""
-    pat = url_data.get_intern_pattern()
-    if pat:
-        log.debug(LOG_CHECK, "Add intern pattern %r", pat)
-        config['internlinks'].append(get_link_pat(pat))
-
 
 def get_link_pat (arg, strict=False):
     """Get a link pattern matcher for intern/extern links.
@@ -99,7 +118,7 @@ def get_link_pat (arg, strict=False):
     @return: dictionary with keys 'pattern', 'negate' and 'strict'
     @rtype: dict
     """
-    log.debug(LOG_CHECK, "Link pattern %r", arg)
+    log.debug(LOG_CHECK, "Link pattern %r strict=%s", arg, strict)
     if arg.startswith('!'):
         pattern = arg[1:]
         negate = True
@@ -113,7 +132,7 @@ def get_link_pat (arg, strict=False):
     }
 
 
-def init_i18n ():
+def init_i18n (loc=None):
     """Initialize i18n with the configured locale dir. The environment
     variable LOCPATH can also specify a locale dir.
 
@@ -123,7 +142,7 @@ def init_i18n ():
         locdir = os.environ['LOCPATH']
     else:
         locdir = os.path.join(get_install_data(), 'share', 'locale')
-    i18n.init(configdata.name, locdir)
+    i18n.init(configdata.name.lower(), locdir, loc=loc)
     # install translated log level names
     import logging
     logging.addLevelName(logging.CRITICAL, _('CRITICAL'))
@@ -134,5 +153,26 @@ def init_i18n ():
     logging.addLevelName(logging.DEBUG, _('DEBUG'))
     logging.addLevelName(logging.NOTSET, _('NOTSET'))
 
-# initialize i18n, puts _() function into global namespace
+# initialize i18n, puts _() and _n() function into global namespace
 init_i18n()
+
+
+def drop_privileges ():
+    """Make sure to drop root privileges on POSIX systems."""
+    if os.name != 'posix':
+        return
+    if os.geteuid() == 0:
+        log.warn(LOG_CHECK, _("Running as root user; "
+                       "dropping privileges by changing user to nobody."))
+        import pwd
+        os.seteuid(pwd.getpwnam('nobody')[3])
+
+
+def find_third_party_modules ():
+    """Find third party modules and add them to the python path."""
+    parent = os.path.dirname(os.path.dirname(__file__))
+    third_party = os.path.join(parent, "third_party")
+    if os.path.isdir(third_party):
+        sys.path.append(os.path.join(third_party, "dnspython"))
+
+find_third_party_modules()
