@@ -22,11 +22,28 @@ import os
 import cgi
 import logging
 import urllib
-from .. import strformat, url as urlutil
+from .. import strformat, url as urlutil, log, LOG_CHECK
 
 MAX_FILESIZE = 1024*1024*10 # 10MB
 
 
+def guess_url(url):
+    """Guess if URL is a http or ftp URL.
+    @param url: the URL to check
+    @ptype url: unicode
+    @return: url with http:// or ftp:// prepended if it's detected as
+      a http respective ftp URL.
+    @rtype: unicode
+    """
+    if url.lower().startswith("www."):
+        # syntactic sugar
+        return "http://%s" % url
+    elif url.lower().startswith("ftp."):
+        # syntactic sugar
+        return "ftp://%s" % url
+    return url
+
+
 def absolute_url (base_url, base_ref, parent_url):
     """
     Search for the absolute url to detect the link type. This does not
@@ -50,7 +67,7 @@ def absolute_url (base_url, base_ref, parent_url):
 
 def get_url_from (base_url, recursion_level, aggregate,
                   parent_url=None, base_ref=None, line=0, column=0,
-                  name=u"", parent_content_type=None):
+                  name=u"", parent_content_type=None, extern=None):
     """
     Get url data from given base data.
 
@@ -70,6 +87,8 @@ def get_url_from (base_url, recursion_level, aggregate,
     @type column: number
     @param name: link name
     @type name: string
+    @param extern: (is_extern, is_strict) or None
+    @type extern: tuple(int, int) or None
     """
     if base_url is not None:
         base_url = strformat.unicode_safe(base_url)
@@ -87,18 +106,22 @@ def get_url_from (base_url, recursion_level, aggregate,
         # use filename as base url, with slash as path seperator
         name = base_url.replace("\\", "/")
     if parent_content_type == 'application/x-httpd-php' and \
-       '<?' in base_url and url.startswith('file:'):
-        # ignore URLs from local PHP files with execution directives
+       '<?' in base_url and '?>' in base_url and url.startswith('file:'):
+        # ignore but warn about URLs from local PHP files with execution directives
         klass = ignoreurl.IgnoreUrl
     else:
-        klass = get_urlclass_from(url)
+        assume_local_file = recursion_level == 0
+        klass = get_urlclass_from(url, assume_local_file=assume_local_file)
+    log.debug(LOG_CHECK, "%s handles url %s", klass.__name__, base_url)
     return klass(base_url, recursion_level, aggregate,
                  parent_url=parent_url, base_ref=base_ref,
-                 line=line, column=column, name=name)
+                 line=line, column=column, name=name, extern=extern)
 
 
-def get_urlclass_from (url):
-    """Return checker class for given URL."""
+def get_urlclass_from (url, assume_local_file=False):
+    """Return checker class for given URL. If URL does not start
+    with a URL scheme and assume_local_file is True, assume that
+    the given URL is a local file."""
     if url.startswith("http:"):
         klass = httpurl.HttpUrl
     elif url.startswith("ftp:"):
@@ -114,11 +137,11 @@ def get_urlclass_from (url):
     elif url.startswith(("nntp:", "news:", "snews:")):
         klass = nntpurl.NntpUrl
     elif unknownurl.is_unknown_url(url):
-        # unknown url
         klass = unknownurl.UnknownUrl
-    else:
-        # assume local file
+    elif assume_local_file:
         klass = fileurl.FileUrl
+    else:
+        klass = unknownurl.UnknownUrl
     return klass
 
 
