@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,13 +36,13 @@ class ProxySupport (object):
         if not self.proxy:
             return
         proxyargs = {"proxy": self.proxy}
+        self.proxytype, self.proxy = urllib.splittype(self.proxy)
         if self.proxytype not in ('http', 'https'):
             # Note that invalid proxies might raise TypeError in urllib2,
             # so make sure to stop checking at this point, not later.
             msg = _("Proxy value `%(proxy)s' must start with 'http:' or 'https:'.") \
                  % proxyargs
             raise LinkCheckerError(msg)
-        self.proxytype, self.proxy = urllib.splittype(self.proxy)
         self.proxy = urllib.splithost(self.proxy)[0]
         self.proxyauth, self.proxy = urllib.splituser(self.proxy)
         if self.ignore_proxy_host():
@@ -71,6 +71,21 @@ class ProxySupport (object):
                     return True
         return False
 
+    def get_netloc(self):
+        """Determine scheme, host and port for this connection taking
+        proxy data into account.
+        @return: tuple (scheme, host, port)
+        @rtype: tuple(string, string, int)
+        """
+        if self.proxy:
+            scheme = self.proxytype
+            host, port = urlutil.splitport(self.proxy)
+        else:
+            scheme = self.scheme
+            host = self.host
+            port = self.port
+        return (scheme, host, port)
+
 
 def parse_host_port (host_port):
     """Parse a host:port string into separate components."""
