@@ -31,7 +31,7 @@ import select
 from . import absolute_url, StoringHandler, get_url_from
 from .. import (log, LOG_CHECK, LOG_CACHE, httputil, httplib2 as httplib,
   strformat, LinkCheckerError, url as urlutil, trace, clamav, winutil, geoip,
-  fileutil)
+  fileutil, get_link_pat)
 from ..HtmlParser import htmlsax
 from ..htmlutil import linkparse
 from ..network import iputil
@@ -40,18 +40,23 @@ from .const import (WARN_URL_EFFECTIVE_URL,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
     WARN_URL_CONTENT_SIZE_TOO_LARGE, WARN_URL_CONTENT_SIZE_ZERO,
     WARN_URL_CONTENT_SIZE_UNEQUAL, WARN_URL_WHITESPACE,
+    WARN_URL_TOO_LONG, URL_MAX_LENGTH, URL_WARN_LENGTH,
+    WARN_URL_CONTENT_DUPLICATE,
     ExcList, ExcSyntaxList, ExcNoCacheList)
 
 # helper alias
 unicode_safe = strformat.unicode_safe
 
-def urljoin (parent, url, scheme):
+# schemes that are invalid with an empty hostname
+scheme_requires_host = ("ftp", "http", "telnet")
+
+def urljoin (parent, url):
     """
     If url is relative, join parent and url. Else leave url as-is.
 
     @return joined url
     """
-    if url.startswith(scheme+":"):
+    if urlutil.url_is_absolute(url):
         return url
     return urlparse.urljoin(parent, url)
 
@@ -96,6 +101,7 @@ class UrlBase (object):
         "text/plain+opera": "opera",
         "text/plain+chromium": "chromium",
         "application/x-plist+safari": "safari",
+        "text/vnd.wap.wml": "wml",
     }
 
     # Set maximum file size for downloaded files in bytes.
@@ -103,7 +109,7 @@ class UrlBase (object):
 
     def __init__ (self, base_url, recursion_level, aggregate,
                   parent_url=None, base_ref=None, line=-1, column=-1,
-                  name=u"", url_encoding=None):
+                  name=u"", url_encoding=None, extern=None):
         """
         Initialize check data, and store given variables.
 
@@ -116,22 +122,23 @@ class UrlBase (object):
         @param column: column number of url in parent content
         @param name: name of url or empty
         @param url_encoding: encoding of URL or None
+        @param extern: None or (is_extern, is_strict)
         """
         self.reset()
         self.init(base_ref, base_url, parent_url, recursion_level,
-                  aggregate, line, column, name, url_encoding)
+                  aggregate, line, column, name, url_encoding, extern)
         self.check_syntax()
+        if recursion_level == 0:
+            self.add_intern_pattern()
+        self.set_extern(self.url)
 
     def init (self, base_ref, base_url, parent_url, recursion_level,
-              aggregate, line, column, name, url_encoding):
+              aggregate, line, column, name, url_encoding, extern):
         """
         Initialize internal data.
         """
         self.base_ref = base_ref
         self.base_url = base_url.strip() if base_url else base_url
-        if self.base_url != base_url:
-            self.add_warning(_("Leading or trailing whitespace in URL `%(url)s'.") %
-                               {"url": base_url}, tag=WARN_URL_WHITESPACE)
         self.parent_url = parent_url
         self.recursion_level = recursion_level
         self.aggregate = aggregate
@@ -140,6 +147,7 @@ class UrlBase (object):
         self.name = name
         self.encoding = url_encoding
         self.charset = None
+        self.extern = extern
         if self.base_ref:
             assert not urlutil.url_needs_quoting(self.base_ref), \
                    "unquoted base reference URL %r" % self.base_ref
@@ -148,7 +156,10 @@ class UrlBase (object):
                    "unquoted parent URL %r" % self.parent_url
         url = absolute_url(self.base_url, base_ref, parent_url)
         # assume file link if no scheme is found
-        self.scheme = url.split(":", 1)[0] or "file"
+        self.scheme = url.split(":", 1)[0].lower() or "file"
+        if self.base_url != base_url:
+            self.add_warning(_("Leading or trailing whitespace in URL `%(url)s'.") %
+                               {"url": base_url}, tag=WARN_URL_WHITESPACE)
 
     def reset (self):
         """
@@ -161,15 +172,13 @@ class UrlBase (object):
         self.url = None
         # a splitted version of url for convenience
         self.urlparts = None
-        # the anchor part of url
-        self.anchor = None
+        # the scheme, host, port and anchor part of url
+        self.scheme = self.host = self.port = self.anchor = None
         # list of parsed anchors
         self.anchors = []
         # the result message string and flag
         self.result = u""
         self.has_result = False
-        # cached or not
-        self.cached = False
         # valid or not
         self.valid = True
         # list of warnings (without duplicates)
@@ -178,6 +187,8 @@ class UrlBase (object):
         self.info = []
         # content size
         self.size = -1
+        # last modification time of content in HTTP-date format as specified in RFC2616 chapter 3.3.1
+        self.modified = None
         # download time
         self.dltime = -1
         # download size
@@ -191,8 +202,8 @@ class UrlBase (object):
         # cache keys, are set by build_url() calling set_cache_keys()
         self.cache_url_key = None
         self.cache_content_key = None
-        # extern flags (is_extern, is_strict), both enabled as default
-        self.extern = (1, 1)
+        # extern flags (is_extern, is_strict)
+        self.extern = None
         # flag if the result should be cached
         self.caching = True
         # title is either the URL or parsed from content
@@ -201,6 +212,8 @@ class UrlBase (object):
         self.do_check_content = True
         # MIME content type
         self.content_type = None
+        # number of URLs in page content
+        self.num_urls = 0
 
     def set_result (self, msg, valid=True, overwrite=False):
         """
@@ -250,7 +263,7 @@ class UrlBase (object):
         try:
             parser.feed(self.get_content())
             parser.flush()
-        except linkparse.StopParse, msg:
+        except linkparse.StopParse as msg:
             log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
@@ -286,12 +299,17 @@ class UrlBase (object):
         """
         return False
 
+    def is_local(self):
+        """Return True for local (ie. file://) URLs."""
+        return self.is_file()
+
     def add_warning (self, s, tag=None):
         """
         Add a warning string.
         """
         item = (tag, s)
-        if item not in self.warnings:
+        if item not in self.warnings and \
+           tag not in self.aggregate.config["ignorewarnings"]:
             self.warnings.append(item)
 
     def add_info (self, s):
@@ -313,7 +331,7 @@ class UrlBase (object):
             # do not copy anchor warnings, since the current anchor
             # might have changed
             if anchor_changed and tag == WARN_URL_ANCHOR_NOT_FOUND:
-                pass
+                continue
             self.add_warning(msg, tag=tag)
         for info in cache_data["info"]:
             self.add_info(info)
@@ -322,12 +340,9 @@ class UrlBase (object):
         self.dlsize = cache_data["dlsize"]
         self.anchors = cache_data["anchors"]
         self.content_type = cache_data["content_type"]
-        self.cached = True
         if anchor_changed and self.valid:
             # recheck anchor
-            if self.check_anchor():
-                # a new warning has been added - remove cached flag
-                self.cached = False
+            self.check_anchor()
 
     def get_cache_data (self):
         """Return all data values that should be put in the cache."""
@@ -343,23 +358,21 @@ class UrlBase (object):
                 "content_type": self.get_content_type(),
                }
 
-    def get_alias_cache_data (self):
-        """Return all data values that should be put in the cache.
-        Intended to be overridden by subclasses that handle aliases.
-        """
-        return self.get_cache_data()
-
     def set_cache_keys (self):
         """
         Set keys for URL checking and content recursion.
         """
         # remove anchor from content cache key since we assume
         # URLs with different anchors to have the same content
-        self.cache_content_key = urlparse.urlunsplit(self.urlparts[:4]+[u''])
+        self.cache_content_key = urlutil.urlunsplit(self.urlparts[:4]+[u''])
         assert isinstance(self.cache_content_key, unicode), self
         log.debug(LOG_CACHE, "Content cache key %r", self.cache_content_key)
         # construct cache key
-        self.cache_url_key = self.cache_content_key
+        if self.aggregate.config["anchors"]:
+            # add anchor to cache key
+            self.cache_url_key = urlutil.urlunsplit(self.urlparts[:4]+[self.anchor or u""])
+        else:
+            self.cache_url_key = self.cache_content_key
         assert isinstance(self.cache_url_key, unicode), self
         log.debug(LOG_CACHE, "URL cache key %r", self.cache_url_key)
 
@@ -379,17 +392,27 @@ class UrlBase (object):
             return
         try:
             self.build_url()
-            # check url warnings
-            effectiveurl = urlparse.urlunsplit(self.urlparts)
-            if self.url != effectiveurl:
-                self.add_warning(_("Effective URL %(url)r.") %
-                                 {"url": effectiveurl},
-                                 tag=WARN_URL_EFFECTIVE_URL)
-                self.url = effectiveurl
-        except tuple(ExcSyntaxList), msg:
+            self.check_url_warnings()
+        except tuple(ExcSyntaxList) as msg:
             self.set_result(unicode_safe(msg), valid=False)
-            return
-        self.set_cache_keys()
+        else:
+            self.set_cache_keys()
+
+    def check_url_warnings(self):
+        """Check URL name and length."""
+        effectiveurl = urlutil.urlunsplit(self.urlparts)
+        if self.url != effectiveurl:
+            self.add_warning(_("Effective URL %(url)r.") %
+                             {"url": effectiveurl},
+                             tag=WARN_URL_EFFECTIVE_URL)
+            self.url = effectiveurl
+        if len(self.url) > URL_MAX_LENGTH:
+            args = dict(len=len(self.url), max=URL_MAX_LENGTH)
+            self.set_result(_("URL length %(len)d is longer than maximum of %(max)d.") % args, valid=False)
+        elif len(self.url) > URL_WARN_LENGTH:
+            args = dict(len=len(self.url), warn=URL_WARN_LENGTH)
+            self.add_warning(_("URL length %(len)d is longer than %(warn)d.") % args,
+                tag=WARN_URL_TOO_LONG)
 
     def build_url (self):
         """
@@ -403,26 +426,31 @@ class UrlBase (object):
             # use base reference as parent url
             if ":" not in self.base_ref:
                 # some websites have a relative base reference
-                self.base_ref = urljoin(self.parent_url, self.base_ref,
-                                        self.scheme)
-            self.url = urljoin(self.base_ref, base_url, self.scheme)
+                self.base_ref = urljoin(self.parent_url, self.base_ref)
+            self.url = urljoin(self.base_ref, base_url)
         elif self.parent_url:
             # strip the parent url query and anchor
             urlparts = list(urlparse.urlsplit(self.parent_url))
             urlparts[4] = ""
-            parent_url = urlparse.urlunsplit(urlparts)
-            self.url = urljoin(parent_url, base_url, self.scheme)
+            parent_url = urlutil.urlunsplit(urlparts)
+            self.url = urljoin(parent_url, base_url)
         else:
             self.url = base_url
-        # note: urljoin can unnorm the url path, so norm it again
+        # urljoin can unnorm the url path, so norm it again
         urlparts = list(urlparse.urlsplit(self.url))
         if urlparts[2]:
             urlparts[2] = urlutil.collapse_segments(urlparts[2])
-        self.url = urlparse.urlunsplit(urlparts)
+        self.url = urlutil.urlunsplit(urlparts)
         # split into (modifiable) list
         self.urlparts = strformat.url_unicode_split(self.url)
         # and unsplit again
-        self.url = urlparse.urlunsplit(self.urlparts)
+        self.url = urlutil.urlunsplit(self.urlparts)
+        self.build_url_parts()
+
+    def build_url_parts (self):
+        """Set userinfo, host, port and anchor from self.urlparts.
+        Also checks for obfuscated IP addresses.
+        """
         # check userinfo@host:port syntax
         self.userinfo, host = urllib.splituser(self.urlparts[1])
         # set host lowercase
@@ -432,19 +460,19 @@ class UrlBase (object):
             self.urlparts[1] = host.lower()
         # safe anchor for later checking
         self.anchor = self.urlparts[4]
-        self.host, self.port = urllib.splitport(host)
-        if self.port is not None:
-            if not urlutil.is_numeric_port(self.port):
-                raise LinkCheckerError(_("URL has invalid port %(port)r") %
-                    {"port": str(self.port)})
-            self.port = int(self.port)
-        self.check_obfuscated_ip()
+        port = urlutil.default_ports.get(self.scheme, 0)
+        self.host, self.port = urlutil.splitport(host, port=port)
+        if self.port is None:
+            raise LinkCheckerError(_("URL host %(host)r has invalid port") %
+                    {"host": host})
+        if self.scheme in scheme_requires_host:
+            if not self.host:
+                raise LinkCheckerError(_("URL has empty hostname"))
+            self.check_obfuscated_ip()
 
     def check_obfuscated_ip (self):
         """Warn if host of this URL is obfuscated IP address."""
         # check if self.host can be an IP address
-        if self.scheme not in ("ftp", "http", "mailto", "news", "nntp", "telnet"):
-            return
         # check for obfuscated IP address
         if iputil.is_obfuscated_ip(self.host):
             ips = iputil.resolve_host(self.host)
@@ -490,11 +518,8 @@ class UrlBase (object):
         log.debug(LOG_CHECK, "Checking %s", self)
         # start time for check
         check_start = time.time()
-        self.set_extern(self.url)
-        if self.extern[0] and self.extern[1]:
-            self.add_info(_("Outside of domain filter, checked only syntax."))
-            return
-
+        # strict extern URLs should not be checked
+        assert not self.extern[1], 'checking strict extern URL'
         # check connection
         log.debug(LOG_CHECK, "checking connection")
         try:
@@ -521,7 +546,10 @@ class UrlBase (object):
                 self.check_size()
             except tuple(ExcList):
                 value = self.handle_exception()
-                self.add_warning(_("could not get content: %(msg)r") %
+                # make nicer error msg for bad status line
+                if isinstance(value, httplib.BadStatusLine):
+                    value = _('Bad HTTP response %(line)r') % {"line": str(value)}
+                self.add_warning(_("could not get content: %(msg)s") %
                      {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
 
     def close_connection (self):
@@ -542,18 +570,19 @@ class UrlBase (object):
         """
         An exception occurred. Log it and set the cache flag.
         """
-        etype, value = sys.exc_info()[:2]
-        log.debug(LOG_CHECK, "Error in %s: %s %s", self.url, etype, value, exception=True)
+        etype, evalue = sys.exc_info()[:2]
+        log.debug(LOG_CHECK, "Error in %s: %s %s", self.url, etype, evalue, exception=True)
         # note: etype must be the exact class, not a subclass
         if (etype in ExcNoCacheList) or \
-           (etype == socket.error and value.args[0]==errno.EBADF) or \
-            not value:
+           (etype == socket.error and evalue.args[0]==errno.EBADF) or \
+            not evalue:
             # EBADF occurs when operating on an already socket
             self.caching = False
-        errmsg = etype.__name__
-        if str(value):
-            # use Exception class name
-            errmsg += ": %s" % str(value)
+        # format unicode message "<exception name>: <error message>"
+        errmsg = unicode(etype.__name__)
+        uvalue = strformat.unicode_safe(evalue)
+        if uvalue:
+            errmsg += u": %s" % uvalue
         # limit length to 240
         return strformat.limit(errmsg, length=240)
 
@@ -611,7 +640,7 @@ class UrlBase (object):
         try:
             parser.feed(self.get_content())
             parser.flush()
-        except linkparse.StopParse, msg:
+        except linkparse.StopParse as msg:
             log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
@@ -622,15 +651,23 @@ class UrlBase (object):
         """Store anchors for this URL. Precondition: this URL is
         an HTML resource."""
         log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
-        handler = linkparse.LinkFinder(self.add_anchor,
-                                   tags={'a': [u'name'], None: [u'id']})
+        self.find_links(self.add_anchor, tags=linkparse.AnchorTags)
+
+    def find_links (self, callback, tags=None):
+        """Parse into content and search for URLs to check.
+        Found URLs are added to the URL queue.
+        """
+        # construct parser object
+        handler = linkparse.LinkFinder(callback, tags=tags)
         parser = htmlsax.parser(handler)
+        if self.charset:
+            parser.encoding = self.charset
         handler.parser = parser
         # parse
         try:
             parser.feed(self.get_content())
             parser.flush()
-        except linkparse.StopParse, msg:
+        except linkparse.StopParse as msg:
             log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
@@ -648,10 +685,14 @@ class UrlBase (object):
                 self.valid and self.is_html()):
             return
         log.debug(LOG_CHECK, "checking anchor %r in %s", self.anchor, self.anchors)
-        if any(x for x in self.anchors if x[0] == self.anchor):
+        enc = lambda anchor: urlutil.url_quote_part(anchor, encoding=self.encoding)
+        if any(x for x in self.anchors if enc(x[0]) == self.anchor):
             return
-        anchornames = sorted(set(u"`%s'" % x[0] for x in self.anchors))
-        anchors = u", ".join(anchornames)
+        if self.anchors:
+            anchornames = sorted(set(u"`%s'" % x[0] for x in self.anchors))
+            anchors = u", ".join(anchornames)
+        else:
+            anchors = u"-"
         args = {"name": self.anchor, "anchors": anchors}
         msg = u"%s %s" % (_("Anchor `%(name)s' not found.") % args,
                           _("Available anchors: %(anchors)s.") % args)
@@ -666,6 +707,11 @@ class UrlBase (object):
 
         @return: None
         """
+        if self.extern:
+            return
+        if not url:
+            self.extern = (1, 1)
+            return
         for entry in self.aggregate.config["externlinks"]:
             match = entry['pattern'].search(url)
             if (entry['negate'] and not match) or \
@@ -709,9 +755,29 @@ class UrlBase (object):
         Can be overridden in subclasses."""
         if self.size > self.MaxFilesizeBytes:
             raise LinkCheckerError(_("File size too large"))
-        data = self.url_connection.read()
+        data = self.url_connection.read(self.MaxFilesizeBytes+1)
+        if len(data) > self.MaxFilesizeBytes:
+            raise LinkCheckerError(_("File size too large"))
+        if not self.is_local():
+            urls = self.aggregate.add_download_data(self.cache_content_key, data)
+            self.warn_duplicate_content(urls)
         return data, len(data)
 
+    def warn_duplicate_content(self, urls):
+        """If given URL list is not empty, warn about duplicate URL content.
+        @param urls: URLs with duplicate content
+        @ptype urls: list of unicode
+        """
+        if not urls or self.size <= 0:
+            return
+        if urlutil.is_duplicate_content_url(self.url, urls[0]):
+            return
+        args = dict(
+            urls=u",".join(urls),
+            size=_(" with %s") % strformat.strsize(self.size),
+        )
+        self.add_warning(_("Content%(size)s is the same as in URLs (%(urls)s).") % args, tag=WARN_URL_CONTENT_DUPLICATE)
+
     def check_content (self):
         """Check content data for warnings, syntax errors, viruses etc."""
         if not (self.valid and self.can_get_content()):
@@ -924,6 +990,7 @@ class UrlBase (object):
         Default parse type is html.
         """
         self.parse_html()
+        self.add_num_url_info()
 
     def get_user_password (self):
         """Get tuple (user, password) from configured authentication.
@@ -939,24 +1006,11 @@ class UrlBase (object):
         Found URLs are added to the URL queue.
         """
         log.debug(LOG_CHECK, "Parsing HTML %s", self)
-        # construct parser object
-        handler = linkparse.LinkFinder(self.add_url)
-        parser = htmlsax.parser(handler)
-        if self.charset:
-            parser.encoding = self.charset
-        handler.parser = parser
-        # parse
-        try:
-            parser.feed(self.get_content())
-            parser.flush()
-        except linkparse.StopParse, msg:
-            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
-        # break cyclic dependencies
-        handler.parser = None
-        parser.handler = None
+        self.find_links(self.add_url)
 
     def add_url (self, url, line=0, column=0, name=u"", base=None):
         """Queue URL data for checking."""
+        self.num_urls += 1
         if base:
             base_ref = urlutil.url_norm(base)[0]
         else:
@@ -964,7 +1018,16 @@ class UrlBase (object):
         url_data = get_url_from(url, self.recursion_level+1, self.aggregate,
             parent_url=self.url, base_ref=base_ref, line=line, column=column,
             name=name, parent_content_type=self.content_type)
-        self.aggregate.urlqueue.put(url_data)
+        if url_data.has_result or not url_data.extern[1]:
+            # Only queue URLs which have a result or are not strict extern.
+            self.aggregate.urlqueue.put(url_data)
+
+    def add_num_url_info(self):
+        """Add number of URLs parsed to info."""
+        if self.num_urls > 0:
+            attrs = {"num": self.num_urls}
+            msg = _n("%(num)d URL parsed.", "%(num)d URLs parsed.", self.num_urls)
+            self.add_info(msg % attrs)
 
     def parse_opera (self):
         """Parse an opera bookmark file."""
@@ -1041,6 +1104,13 @@ class UrlBase (object):
         except winutil.Error, msg:
             log.warn(LOG_CHECK, "Error parsing word file: %s", msg)
 
+    def parse_wml (self):
+        """Parse into WML content and search for URLs to check.
+        Found URLs are added to the URL queue.
+        """
+        log.debug(LOG_CHECK, "Parsing WML %s", self)
+        self.find_links(self.add_url, tags=linkparse.WmlTags)
+
     def get_temp_filename (self):
         """Get temporary filename for content to parse."""
         # store content in temporary file
@@ -1052,11 +1122,11 @@ class UrlBase (object):
             fd.close()
         return filename
 
-    def serialized (self):
+    def serialized (self, sep=os.linesep):
         """
         Return serialized url check data as unicode string.
         """
-        sep = unicode_safe(os.linesep)
+        sep = unicode_safe(sep)
         if self.base_url is not None:
             assert isinstance(self.base_url, unicode), self
         if self.parent_url is not None:
@@ -1064,28 +1134,46 @@ class UrlBase (object):
         if self.base_ref is not None:
             assert isinstance(self.base_ref, unicode), self
         assert isinstance(self.name, unicode), self
+        if self.anchor is not None:
+            assert isinstance(self.anchor, unicode), self
+        if self.cache_url_key is not None:
+            assert isinstance(self.cache_url_key, unicode), self
         return sep.join([
             u"%s link" % self.scheme,
             u"base_url=%r" % self.base_url,
             u"parent_url=%r" % self.parent_url,
             u"base_ref=%r" % self.base_ref,
-            u"recursion_level=%s" % self.recursion_level,
+            u"recursion_level=%d" % self.recursion_level,
             u"url_connection=%s" % self.url_connection,
             u"line=%d" % self.line,
             u"column=%d" % self.column,
             u"name=%r" % self.name,
             u"anchor=%r" % self.anchor,
+            u"cache_key=%r" % self.cache_url_key,
            ])
 
-    def get_intern_pattern (self):
-        """
-        Get pattern for intern URL matching.
+    def get_intern_pattern (self, url=None):
+        """Get pattern for intern URL matching.
 
+        @param url: the URL to set intern pattern for, else self.url
+        @ptype url: unicode or None
         @return non-empty regex pattern or None
         @rtype String or None
         """
         return None
 
+    def add_intern_pattern(self, url=None):
+        """Add intern URL regex to config."""
+        try:
+            pat = self.get_intern_pattern(url=url)
+            if pat:
+                log.debug(LOG_CHECK, "Add intern pattern %r", pat)
+                self.aggregate.config['internlinks'].append(get_link_pat(pat))
+        except UnicodeError, msg:
+            res = _("URL has unparsable domain name: %(domain)s") % \
+                {"domain": msg}
+            self.set_result(res, valid=False)
+
     def __str__ (self):
         """
         Get URL info.
@@ -1103,7 +1191,7 @@ class UrlBase (object):
         @return: URL info
         @rtype: unicode
         """
-        return u"<%s >" % self.serialized()
+        return u"<%s>" % self.serialized(sep=u", ")
 
     def to_wire_dict (self):
         """Return a simplified transport object for logging.
@@ -1111,8 +1199,6 @@ class UrlBase (object):
         The transport object must contain these attributes:
         - url_data.valid: bool
           Indicates if URL is valid
-        - url_data.cached: bool
-          Indicates if URL data has been loaded from cache.
         - url_data.result: unicode
           Result string
         - url_data.warnings: list of tuples (tag, warning message)
@@ -1145,10 +1231,11 @@ class UrlBase (object):
           MIME content type for URL content.
         - url_data.level: int
           Recursion level until reaching this URL from start URL
+        - url_data.last_modified: datetime
+          Last modification date of retrieved page (or None).
         """
         return dict(valid=self.valid,
           extern=self.extern[0],
-          cached=self.cached,
           result=self.result,
           warnings=self.warnings[:],
           name=self.name or u"",
@@ -1167,6 +1254,7 @@ class UrlBase (object):
           cache_url_key=self.cache_url_key,
           content_type=self.get_content_type(),
           level=self.recursion_level,
+          modified=self.modified,
         )
 
     def to_wire (self):
@@ -1184,7 +1272,6 @@ def filter_tidy_errors (errors):
 urlDataAttr = [
     'valid',
     'extern',
-    'cached',
     'result',
     'warnings',
     'name',
@@ -1198,6 +1285,7 @@ urlDataAttr = [
     'dltime',
     'dlsize',
     'info',
+    'modified',
     'line',
     'column',
     'cache_url_key',
