@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2012 Bastian Kleineidam
+# Copyright (C) 2000-2013 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ import logging.config
 import urllib
 import urlparse
 import shutil
+import socket
 import _LinkChecker_configdata as configdata
 from .. import (log, LOG_CHECK, LOG_ROOT, ansicolor, lognames, clamav,
     get_config_dir, fileutil, configdict)
@@ -37,15 +38,15 @@ AppName = configdata.name
 App = AppName+u" "+Version
 Author = configdata.author
 HtmlAuthor = Author.replace(u' ', u'&nbsp;')
-Copyright = u"Copyright (C) 2000-2012 "+Author
-HtmlCopyright = u"Copyright &copy; 2000-2012 "+HtmlAuthor
+Copyright = u"Copyright (C) 2000-2013 "+Author
+HtmlCopyright = u"Copyright &copy; 2000-2013 "+HtmlAuthor
 AppInfo = App+u"              "+Copyright
 HtmlAppInfo = App+u", "+HtmlCopyright
 Url = configdata.url
-SupportUrl = u"http://sourceforge.net/projects/linkchecker/support"
-DonateUrl = u"http://linkchecker.sourceforge.net/donations.html"
+SupportUrl = u"https://github.com/wummel/linkchecker/issues"
+DonateUrl = u"http://wummel.github.com/linkchecker/donations.html"
 Email = configdata.author_email
-UserAgent = u"%s/%s (+%s)" % (AppName, Version, Url)
+UserAgent = u"Mozilla/5.0 (compatible; %s/%s; +%s)" % (AppName, Version, Url)
 Freeware = AppName+u""" comes with ABSOLUTELY NO WARRANTY!
 This is free software, and you are welcome to redistribute it
 under certain conditions. Look at the file `LICENSE' within this
@@ -70,6 +71,7 @@ Modules = (
     ("twill", u"Twill"),
     ("sqlite3", u"Sqlite"),
     ("gconf", u"Gconf"),
+    ("meliae", u"Meliae"),
 )
 
 def get_modules_info ():
@@ -89,6 +91,32 @@ def get_modules_info ():
     return lines
 
 
+def get_share_file (devel_dir, filename):
+    """Return a filename in the share directory.
+    @param devel_dir: directory to search when developing
+    @ptype devel_dir: string
+    @param filename: filename to search for
+    @ptype filename: string
+    @return: the found filename or None
+    @rtype: string or None
+    """
+    paths = [
+        # when developing
+        devel_dir,
+        # when running under py2exe
+        os.path.join(os.path.dirname(os.path.abspath(sys.executable)),
+                     "share", "linkchecker"),
+        # after installing as a package
+        configdata.config_dir,
+    ]
+    for path in paths:
+        fullpath = os.path.join(path, filename)
+        if os.path.isfile(fullpath):
+            return fullpath
+    # not found
+    return None
+
+
 # dynamic options
 class Configuration (dict):
     """
@@ -177,6 +205,10 @@ class Configuration (dict):
             "filename": "linkchecker-out.dot",
             "encoding": "ascii",
         }
+        self['sitemap'] = {
+            "filename": "linkchecker-out.sitemap.xml",
+            "encoding": "utf-8",
+        }
         self['none'] = {}
         self['output'] = 'text'
         self['logger'] = None
@@ -184,7 +216,7 @@ class Configuration (dict):
         self["warningregex_max"] = 5
         self["warnsizebytes"] = None
         self["nntpserver"] = os.environ.get("NNTP_SERVER", None)
-        self["threads"] = 10
+        self["threads"] = 100
         # socket timeout in seconds
         self["timeout"] = 60
         self["checkhtml"] = False
@@ -194,6 +226,9 @@ class Configuration (dict):
         self["useragent"] = UserAgent
         self["debugmemory"] = False
         self["localwebroot"] = None
+        self["warnsslcertdaysvalid"] = 14
+        self["maxrunseconds"] = None
+        self["maxnumurls"] = None
         from ..logger import Loggers
         self.loggers = dict(**Loggers)
 
@@ -208,7 +243,7 @@ class Configuration (dict):
         logging.config.dictConfig(configdict)
         if handler is None:
             handler = ansicolor.ColoredStreamHandler(strm=sys.stderr)
-        self.add_loghandler(handler)
+        self.add_loghandler(handler, debug)
         self.set_debug(debug)
         self.status_logger = status_logger
 
@@ -217,13 +252,15 @@ class Configuration (dict):
         is a list of logger names to enable debug for."""
         self.set_loglevel(debug, logging.DEBUG)
 
-    def add_loghandler (self, handler):
+    def add_loghandler (self, handler, debug):
         """Add log handler to root logger LOG_ROOT and set formatting."""
         logging.getLogger(LOG_ROOT).addHandler(handler)
+        format = "%(levelname)s "
+        if debug:
+            format += "%(asctime)s "
         if self['threads'] > 0:
-            format = "%(levelname)s %(threadName)s %(message)s"
-        else:
-            format = "%(levelname)s %(message)s"
+            format += "%(threadName)s "
+        format += "%(message)s"
         handler.setFormatter(logging.Formatter(format))
 
     def remove_loghandler (self, handler):
@@ -275,11 +312,20 @@ class Configuration (dict):
         else:
             cfiles = files[:]
         if not cfiles:
-            cfiles.append(get_user_config())
-        # weed out invalid files
-        cfiles = [f for f in cfiles if os.path.isfile(f)]
-        log.debug(LOG_CHECK, "reading configuration from %s", cfiles)
-        confparse.LCConfigParser(self).read(cfiles)
+            userconf = get_user_config()
+            if os.path.isfile(userconf):
+                cfiles.append(userconf)
+        # filter invalid files
+        filtered_cfiles = []
+        for cfile in cfiles:
+            if not os.path.isfile(cfile):
+                log.warn(LOG_CHECK, _("Configuration file %r does not exist."), cfile)
+            elif not fileutil.is_readable(cfile):
+                log.warn(LOG_CHECK, _("Configuration file %r is not readable."), cfile)
+            else:
+                filtered_cfiles.append(cfile)
+        log.debug(LOG_CHECK, "reading configuration from %s", filtered_cfiles)
+        confparse.LCConfigParser(self).read(filtered_cfiles)
         self.sanitize()
 
     def add_auth (self, user=None, password=None, pattern=None):
@@ -319,6 +365,8 @@ class Configuration (dict):
         if self['loginurl']:
             self.sanitize_loginurl()
         self.sanitize_proxies()
+        # set default socket timeout
+        socket.setdefaulttimeout(self['timeout'])
 
     def sanitize_anchors (self):
         """Make anchor configuration consistent."""
@@ -404,8 +452,11 @@ def get_user_config():
     """Get the user configuration filename.
     If the user configuration file does not exist, copy it from the initial
     configuration file, but only if this is not a portable installation.
-    Returns path to user config file (which might not exist do to copy
-    failures or on portable systems)."""
+    Returns path to user config file (which might not exist due to copy
+    failures or on portable systems).
+    @return configuration filename
+    @rtype string
+    """
     # initial config (with all options explained)
     initialconf = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
     # per user config settings
@@ -416,9 +467,13 @@ def get_user_config():
         try:
             userdir = os.path.dirname(userconf)
             if not os.path.isdir(userdir):
+                if os.name == 'nt':
+                    # Windows forbids filenames with leading dot unless
+                    # a trailing dot is added.
+                    userdir += "."
                 os.mkdir(userdir, 0700)
             shutil.copy(initialconf, userconf)
-        except StandardError, errmsg:
+        except StandardError as errmsg:
             msg = _("could not copy initial configuration file %(src)r to %(dst)r: %(errmsg)r")
             args = dict(src=initialconf, dst=userconf, errmsg=errmsg)
             log.warn(LOG_CHECK, msg % args)
@@ -440,7 +495,7 @@ def get_gconf_http_proxy ():
                 if not port:
                     port = 8080
                 return "%s:%d" % (host, port)
-    except StandardError, msg:
+    except StandardError as msg:
         log.debug(LOG_CHECK, "error getting HTTP proxy from gconf: %s", msg)
     return None
 
@@ -459,7 +514,7 @@ def get_gconf_ftp_proxy ():
             if not port:
                 port = 8080
             return "%s:%d" % (host, port)
-    except StandardError, msg:
+    except StandardError as msg:
         log.debug(LOG_CHECK, "error getting FTP proxy from gconf: %s", msg)
     return None
 
@@ -473,7 +528,7 @@ def get_kde_http_proxy ():
     try:
         data = read_kioslaverc(config_dir)
         return data.get("http_proxy")
-    except StandardError, msg:
+    except StandardError as msg:
         log.debug(LOG_CHECK, "error getting HTTP proxy from KDE: %s", msg)
 
 
@@ -486,7 +541,7 @@ def get_kde_ftp_proxy ():
     try:
         data = read_kioslaverc(config_dir)
         return data.get("ftp_proxy")
-    except StandardError, msg:
+    except StandardError as msg:
         log.debug(LOG_CHECK, "error getting FTP proxy from KDE: %s", msg)
 
 # The following KDE functions are largely ported and ajusted from
