@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,8 @@
 
 import ConfigParser
 import re
-from .. import LOG_CHECK, LinkCheckerError, get_link_pat
+import os
+from .. import LinkCheckerError, get_link_pat, LOG_CHECK, log, fileutil
 
 
 def read_multiline (value):
@@ -45,31 +46,61 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
 
         @raises: LinkCheckerError on syntax errors in the config file(s)
         """
+        assert isinstance(files, list), "Invalid file list %r" % files
         try:
-            super(LCConfigParser, self).read(files)
+            self.read_ok = super(LCConfigParser, self).read(files)
+            if len(self.read_ok) < len(files):
+                failed_files = set(files) - set(self.read_ok)
+                log.warn(LOG_CHECK, "Could not read configuration files %s.", failed_files)
             # Read all the configuration parameters from the given files.
             self.read_output_config()
             self.read_checking_config()
             self.read_authentication_config()
             self.read_filtering_config()
-        except Exception, msg:
+        except Exception as msg:
             raise LinkCheckerError(
-              "Error parsing configuration: %s", str(msg))
+              _("Error parsing configuration: %s") % unicode(msg))
+
+    def read_string_option (self, section, option, allowempty=False):
+        """Read a sring option."""
+        if self.has_option(section, option):
+            value = self.get(section, option)
+            if not allowempty and not value:
+                raise LinkCheckerError(_("invalid empty value for %s: %s\n") % (option, value))
+            self.config[option] = value
+
+    def read_boolean_option(self, section, option):
+        """Read a boolean option."""
+        if self.has_option(section, option):
+            self.config[option] = self.getboolean(section, option)
+
+    def read_int_option (self, section, option, key=None, min=None, max=None):
+        """Read an integer option."""
+        if self.has_option(section, option):
+            num = self.getint(section, option)
+            if min is not None and num < min:
+                raise LinkCheckerError(
+                    _("invalid value for %s: %d must not be less than %d") % (option, num, min))
+            if max is not None and num < max:
+                raise LinkCheckerError(
+                    _("invalid value for %s: %d must not be greater than %d") % (option, num, max))
+            if key is None:
+                key = option
+            self.config[key] = num
 
     def read_output_config (self):
         """Read configuration options in section "output"."""
         section = "output"
         from ..logger import Loggers
-        for key in Loggers.iterkeys():
+        for key in Loggers.keys():
             if self.has_section(key):
                 for opt in self.options(key):
                     self.config[key][opt] = self.get(key, opt)
                 if self.has_option(key, 'parts'):
                     val = self.get(key, 'parts')
-                    parts = [f.strip() for f in val.split(',')]
+                    parts = [f.strip().lower() for f in val.split(',')]
                     self.config[key]['parts'] = parts
-        if self.has_option(section, "warnings"):
-            self.config["warnings"] = self.getboolean(section, "warnings")
+        self.read_boolean_option(section, "warnings")
         if self.has_option(section, "verbose"):
             if self.getboolean(section, "verbose"):
                 self.config["verbose"] = True
@@ -85,17 +116,16 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
                 self.config['quiet'] = True
         if self.has_option(section, "debug"):
             val = self.get(section, "debug")
-            parts = [f.strip() for f in val.split(',')]
+            parts = [f.strip().lower() for f in val.split(',')]
             self.config.set_debug(parts)
-        if self.has_option(section, "status"):
-            self.config["status"] = self.getboolean(section, "status")
+        self.read_boolean_option(section, "status")
         if self.has_option(section, "log"):
-            val = self.get(section, "log").strip()
+            val = self.get(section, "log").strip().lower()
             self.config['output'] = val
         if self.has_option(section, "fileoutput"):
             loggers = self.get(section, "fileoutput").split(",")
             # strip names from whitespace
-            loggers = (x.strip() for x in loggers)
+            loggers = (x.strip().lower() for x in loggers)
             # no file output for the blacklist and none Logger
             loggers = (x for x in loggers if x in Loggers and
                        x not in ("blacklist", "none"))
@@ -106,85 +136,86 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
     def read_checking_config (self):
         """Read configuration options in section "checking"."""
         section = "checking"
-        if self.has_option(section, "threads"):
-            num = self.getint(section, "threads")
-            self.config['threads'] = max(0, num)
-        if self.has_option(section, "timeout"):
-            num = self.getint(section, "timeout")
-            if num < 0:
-                raise LinkCheckerError(
-                    _("invalid negative value for timeout: %d\n"), num)
-            self.config['timeout'] = num
-        if self.has_option(section, "anchors"):
-            self.config["anchors"] = self.getboolean(section, "anchors")
-        if self.has_option(section, "recursionlevel"):
-            num = self.getint(section, "recursionlevel")
-            self.config["recursionlevel"] = num
+        self.read_int_option(section, "threads", min=-1)
+        self.config['threads'] = max(0, self.config['threads'])
+        self.read_int_option(section, "timeout", min=1)
+        self.read_boolean_option(section, "anchors")
+        self.read_int_option(section, "recursionlevel", min=-1)
         if self.has_option(section, "warningregex"):
             val = self.get(section, "warningregex")
             if val:
                 self.config["warningregex"] = re.compile(val)
-        if self.has_option(section, "warnsizebytes"):
-            val = self.get(section, "warnsizebytes")
-            self.config["warnsizebytes"] = int(val)
-        if self.has_option(section, "nntpserver"):
-            self.config["nntpserver"] = self.get(section, "nntpserver")
-        if self.has_option(section, "useragent"):
-            self.config["useragent"] = self.get(section, "useragent")
+        self.read_int_option(section, "warnsizebytes", min=1)
+        self.read_string_option(section, "nntpserver")
+        self.read_string_option(section, "useragent")
+        self.read_int_option(section, "pause", key="wait", min=0)
         self.read_check_options(section)
 
     def read_check_options (self, section):
         """Read check* options."""
-        if self.has_option(section, "checkhtml"):
-            self.config["checkhtml"] = self.getboolean(section, "checkhtml")
-        if self.has_option(section, "checkcss"):
-            self.config["checkcss"] = self.getboolean(section, "checkcss")
-        if self.has_option(section, "checkhtmlw3"):
-            self.config["checkhtmlw3"] = \
-               self.getboolean(section, "checkhtmlw3")
-        if self.has_option(section, "checkcssw3"):
-            self.config["checkcssw3"] = self.getboolean(section, "checkcssw3")
-        if self.has_option(section, "scanvirus"):
-            self.config["scanvirus"] = self.getboolean(section, "scanvirus")
-        if self.has_option(section, "clamavconf"):
-            self.config["clamavconf"] = self.getboolean(section, "clamavconf")
+        self.read_boolean_option(section, "checkhtml")
+        self.read_boolean_option(section, "checkcss")
+        self.read_boolean_option(section, "scanvirus")
+        self.read_boolean_option(section, "clamavconf")
+        self.read_boolean_option(section, "debugmemory")
         if self.has_option(section, "cookies"):
             self.config["sendcookies"] = self.config["storecookies"] = \
                 self.getboolean(section, "cookies")
+        self.read_string_option(section, "cookiefile")
+        self.read_string_option(section, "localwebroot")
+        self.read_int_option(section, "warnsslcertdaysvalid", min=1)
+        self.read_int_option(section, "maxrunseconds", min=0)
 
     def read_authentication_config (self):
         """Read configuration options in section "authentication"."""
         section = "authentication"
+        password_fields = []
         if self.has_option(section, "entry"):
             for val in read_multiline(self.get(section, "entry")):
                 auth = val.split()
                 if len(auth) == 3:
                     self.config.add_auth(pattern=auth[0], user=auth[1],
                                          password=auth[2])
+                    password_fields.append("entry/%s/%s" % (auth[0], auth[1]))
                 elif len(auth) == 2:
                     self.config.add_auth(pattern=auth[0], user=auth[1])
                 else:
-                    raise LinkCheckerError(LOG_CHECK,
-                       _("missing auth part in entry %(val)r") % \
-                       {"val": val})
+                    raise LinkCheckerError(
+                       _("missing auth part in entry %(val)r") % {"val": val})
         # read login URL and field names
         if self.has_option(section, "loginurl"):
             val = self.get(section, "loginurl").strip()
             if not (val.lower().startswith("http:") or
                     val.lower().startswith("https:")):
-                raise LinkCheckerError(LOG_CHECK,
-                _("Invalid login URL `%s'. Only " \
+                raise LinkCheckerError(_("invalid login URL `%s'. Only " \
                   "HTTP and HTTPS URLs are supported.") % val)
             self.config["loginurl"] = val
             self.config["storecookies"] = self.config["sendcookies"] = True
-        for key in ("loginuserfield", "loginpasswordfield"):
-            if self.has_option(section, key):
-                self.config[key] = self.get(section, key)
+        self.read_string_option(section, "loginuserfield")
+        self.read_string_option(section, "loginpasswordfield")
         # read login extra fields
         if self.has_option(section, "loginextrafields"):
             for val in read_multiline(self.get(section, "loginextrafields")):
                 name, value = val.split(":", 1)
                 self.config["loginextrafields"][name] = value
+        self.check_password_readable(section, password_fields)
+
+    def check_password_readable(self, section, fields):
+        """Check if there is a readable configuration file and print a warning."""
+        if not fields:
+            return
+        # The information which of the  configuration files
+        # included which option is not available. To avoid false positives,
+        # a warning is only printed if exactly one file has been read.
+        if len(self.read_ok) != 1:
+            return
+        fn = self.read_ok[0]
+        if fileutil.is_accessable_by_others(fn):
+            log.warn(LOG_CHECK, "The configuration file %s contains password information (in section [%s] and options %s) and the file is readable by others. Please make the file only readable by you.", fn, section, fields)
+            if os.name == 'posix':
+                log.warn(LOG_CHECK, _("For example execute 'chmod go-rw %s'.") % fn)
+            elif os.name == 'nt':
+                log.warn(LOG_CHECK, _("See http://support.microsoft.com/kb/308419 for more info on setting file permissions."))
 
     def read_filtering_config (self):
         """
