@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2008-2011 Bastian Kleineidam
+# Copyright (C) 2008-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,14 @@
 
 import os
 import sys
+import re
 import webbrowser
 from PyQt4 import QtCore, QtGui
 from .linkchecker_ui_main import Ui_MainWindow
 from .properties import set_properties, clear_properties
 from .statistics import set_statistics, clear_statistics
-from .debug import LinkCheckerDebug
-from .logger import GuiLogger, GuiLogHandler, StatusLogger
+from .debug import LinkCheckerDebug, LinkCheckerDebugMemory
+from .logger import SignalLogger, GuiLogHandler, StatusLogger
 from .help import HelpWindow
 from .options import LinkCheckerOptions
 from .checker import CheckerThread
@@ -34,8 +35,9 @@ from .urlmodel import UrlItemModel
 from .urlsave import urlsave
 from .settings import Settings
 from .recentdocs import RecentDocumentModel
-from .. import configuration, checker, director, add_intern_pattern, \
-    strformat, fileutil, LinkCheckerError
+from .projects import openproject, saveproject, loadproject, ProjectExt
+from .. import configuration, checker, director, get_link_pat, \
+    strformat, fileutil, LinkCheckerError, memoryutil
 from ..containers import enum
 from .. import url as urlutil
 from ..checker import httpheaders
@@ -45,6 +47,7 @@ DocBaseUrl = "qthelp://bfk.app.linkchecker/doc/"
 RegistryBase = "Bastian"
 Status = enum('idle', 'checking')
 
+MaxMessageLength = 60
 
 def get_app_style ():
     """Return appropriate QStyle object for the current platform to
@@ -71,6 +74,12 @@ def get_icon (name):
     return icon
 
 
+def warninglines2regex(lines):
+    """Convert a list of strings to a regular expression matching any of
+    the given strings."""
+    return u"|".join([re.escape(line) for line in lines])
+
+
 class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     """The main window displaying checked URLs."""
 
@@ -79,7 +88,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     log_stats_signal = QtCore.pyqtSignal(object)
     error_signal = QtCore.pyqtSignal(str)
 
-    def __init__(self, parent=None, url=None):
+    def __init__(self, parent=None, url=None, project=None):
         """Initialize UI."""
         super(LinkCheckerMain, self).__init__(parent)
         self.setupUi(self)
@@ -90,6 +99,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         # init subdialogs
         self.options = LinkCheckerOptions(parent=self)
         self.debug = LinkCheckerDebug(parent=self)
+        self.debugmemory = LinkCheckerDebugMemory(parent=self)
         self.checker = CheckerThread(parent=self)
         self.contextmenu = ContextMenu(parent=self)
         self.editor = EditorWindow(parent=self)
@@ -106,9 +116,12 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.init_url(url)
         self.init_treeview()
         self.connect_widgets()
+        self.init_shortcuts()
         self.init_config()
         self.read_config()
-        self.init_app()
+        self.init_menu()
+        self.init_drop()
+        self.init_app(project)
 
     def init_url (self, url):
         """Initialize URL input."""
@@ -120,7 +133,18 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         elif documents:
             self.urlinput.setText(documents[0])
 
-    def init_app (self):
+    def init_menu (self):
+        """Add menu entries for bookmark file checking."""
+        self.urlinput.addMenuEntries(self.menuEdit)
+
+    def init_drop(self):
+        """Set and activate drag-and-drop functions."""
+        self.__class__.dragEnterEvent = self.handleDragEvent
+        self.__class__.dragMoveEvent = self.handleDragEvent
+        self.__class__.dropEvent = self.handleDropEvent
+        self.setAcceptDrops(True)
+
+    def init_app (self, project):
         """Set window size and position, GUI options and reset status."""
         data = self.settings.read_geometry()
         if data["size"] is not None:
@@ -130,25 +154,17 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.options.set_options(self.settings.read_options())
         self.status = Status.idle
         self.actionSave.setEnabled(False)
-        msg = self.config_error or _("Ready.")
-        self.set_statusmsg(msg)
+        if project:
+            loadproject(self, project)
+        else:
+            msg = self.config_error or _("Ready.")
+            self.set_statusmsg(msg)
 
     def get_qhcpath (self):
         """Helper function to search for the QHC help file in different
         locations."""
-        paths = [
-            # when developing
-            os.path.join(configuration.configdata.install_data, "doc", "html"),
-            # when running under py2exe
-            os.path.join(os.path.dirname(os.path.abspath(sys.executable)), "share", "linkchecker"),
-            # after installing as a package
-            configuration.configdata.config_dir,
-        ]
-        for path in paths:
-            qhcfile = os.path.join(path, "lccollection.qhc")
-            if os.path.isfile(qhcfile):
-                break
-        return qhcfile
+        devel_dir = os.path.join(configuration.configdata.install_data, "doc", "html")
+        return configuration.get_share_file(devel_dir, 'lccollection.qhc')
 
     def connect_widgets (self):
         """Connect widget signals. Some signals use the AutoConnect feature.
@@ -157,6 +173,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         def set_idle ():
             """Set application status to idle."""
             self.status = Status.idle
+            if self.config["debugmemory"]:
+                self.dump_memory()
             self.set_statusmsg(_("Check finished."))
             self.controlButton.clicked.disconnect(self.checker.cancel)
         self.checker.finished.connect(set_idle)
@@ -166,6 +184,17 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.error_signal.connect(self.internal_error)
         self.options.editor.saved.connect(self.read_config)
         self.log_status_signal.connect(self.log_status)
+        self.prop_url.linkHovered.connect(self.hover_link)
+        self.prop_parenturl.linkHovered.connect(self.hover_link)
+
+    def init_shortcuts (self):
+        """Configure application shortcuts."""
+        def selectUrl():
+            """Highlight URL input textbox."""
+            self.urlinput.setFocus()
+            self.urlinput.selectAll()
+        shortcut = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+L"), self)
+        shortcut.activated.connect(selectUrl)
 
     def init_treeview (self):
         """Set treeview model and layout."""
@@ -189,8 +218,10 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     def init_config (self):
         """Create a configuration object."""
         self.config = configuration.Configuration()
+        # dictionary holding overwritten values
+        self.config_backup = {}
         # set standard GUI configuration values
-        self.config.logger_add("gui", GuiLogger)
+        self.config.logger_add("gui", SignalLogger)
         self.config["logger"] = self.config.logger_new('gui',
             signal=self.log_url_signal, stats=self.log_stats_signal)
         self.config["status"] = True
@@ -203,8 +234,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         """Read user and system configuration file."""
         try:
             self.config.read()
-        except LinkCheckerError, msg:
-            self.config_error = msg
+        except LinkCheckerError as msg:
+            self.config_error = unicode(msg)
 
     def set_config (self):
         """Set configuration."""
@@ -217,6 +248,39 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.config["threads"] = 1
         else:
             self.config.reset_loglevel()
+        self.config["debugmemory"] = data["debugmemory"]
+        if data["warninglines"]:
+            lines = data["warninglines"].splitlines()
+            ro = re.compile(warninglines2regex(lines))
+            self.backup_config("warningregex", ro)
+        # set ignore patterns
+        ignorepats = data["ignorelines"].strip()
+        if ignorepats:
+            self.backup_config("externlinks")
+            lines = ignorepats.splitlines()
+            for line in lines:
+                pat = get_link_pat(line, strict=1)
+                self.config["externlinks"].append(pat)
+
+    def backup_config (self, key, value=None):
+        """Backup config key if not already done and set given value."""
+        if key not in self.config_backup:
+            confvalue = self.config[key]
+            if isinstance(confvalue, list):
+                # make copy of lists to avoid unwanted inserted items
+                confvalue = confvalue[:]
+            self.config_backup[key] = confvalue
+        if value is not None:
+            self.config[key] = value
+
+    def restore_config (self):
+        """Restore config from backup."""
+        for key in self.config_backup:
+            confvalue = self.config_backup[key]
+            if isinstance(confvalue, list):
+                # make copy of lists to avoid unwanted inserted items
+                confvalue = confvalue[:]
+            self.config[key] = confvalue
 
     def get_status (self):
         """Return current application status."""
@@ -243,9 +307,11 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.label_busy.hide()
             self.menubar.setEnabled(True)
             self.urlinput.setEnabled(True)
+            self.restore_config()
         elif status == Status.checking:
             self.treeView.setSortingEnabled(False)
             self.debug.reset()
+            self.debugmemory.reset()
             self.set_statusmsg(u"Checking site...")
             # disable commands
             self.menubar.setEnabled(False)
@@ -297,6 +363,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             "donateurl": configuration.DonateUrl,
             "pyver": u"%d.%d.%d" % sys.version_info[:3],
             "modules": modules,
+            "portable": _("yes") if configuration.Portable else _("no"),
             "releasedate": configuration.ReleaseDate,
         }
         QtGui.QMessageBox.about(self, _(u"About %(appname)s") % d,
@@ -304,7 +371,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
 <h1>%(app)s</h1>
 <p>Released on %(releasedate)s
 <p>Python: %(pyver)s<br>
-%(modules)s
+%(modules)s<br>
+Portable version: %(portable)s
 <p>%(copyright)s
 <br>%(appname)s is licensed under the
 <a href="http://www.gnu.org/licenses/gpl.html">GPL</a>
@@ -313,19 +381,24 @@ Version 2 or later.
 <a href="%(donateurl)s">donate</a>. Thanks!
 </center></qt>""") % d)
 
-    @QtCore.pyqtSlot()
-    def on_actionDonate_triggered (self):
-        """Open donation URL."""
-        webbrowser.open(configuration.DonateUrl)
-
     @QtCore.pyqtSlot()
     def on_actionDebug_triggered (self):
         """Display debug dialog."""
         self.debug.show()
 
+    @QtCore.pyqtSlot()
+    def on_actionOpen_project_triggered (self):
+        """Open project."""
+        openproject(self)
+
+    @QtCore.pyqtSlot()
+    def on_actionSave_project_triggered (self):
+        """Save project."""
+        saveproject(self, self.get_url())
+
     @QtCore.pyqtSlot()
     def on_actionSave_triggered (self):
-        """Quit application."""
+        """Save URL results."""
         urlsave(self, self.config, self.model.urls)
 
     @QtCore.pyqtSlot()
@@ -345,7 +418,8 @@ Version 2 or later.
     def cancel (self):
         """Note that checking is canceled."""
         self.controlButton.setEnabled(False)
-        self.set_statusmsg(_(u"Closing pending connections..."))
+        duration = strformat.strduration_long(self.config["timeout"])
+        self.set_statusmsg(_(u"Closing active URLs with timeout %s...") % duration)
 
     @QtCore.pyqtSlot()
     def on_controlButton_clicked (self):
@@ -357,14 +431,21 @@ Version 2 or later.
         else:
             raise ValueError("Invalid application status %r" % self.status)
 
+    def dump_memory (self):
+        """Dump memory to temporary file and inform user with a modal
+        dialog where the file is."""
+        self.set_statusmsg(_(u"Dumping memory statistics..."))
+        filename = memoryutil.write_memory_dump()
+        title = _(u"LinkChecker memory dump written")
+        message = _(u"The memory dump has been written to `%(filename)s'.")
+        attrs = dict(filename=filename)
+        QtGui.QMessageBox.information(self, title, message % attrs)
+
     def get_url (self):
         """Return URL to check from the urlinput widget."""
-        url = strformat.stripall(unicode(self.urlinput.text()))
-        if url.startswith(u"www."):
-            url = u"http://%s" % url
-        elif url.startswith(u"ftp."):
-            url = u"ftp://%s" % url
-        elif url and u":" not in url:
+        url = strformat.stripurl(unicode(self.urlinput.text()))
+        url = checker.guess_url(url)
+        if url and u":" not in url:
             # Look for local file, else assume it's an HTTP URL.
             if not os.path.exists(url):
                 url = u"http://%s" % url
@@ -382,13 +463,7 @@ Version 2 or later.
             self.set_statusmsg(_("Error, empty URL"))
             return
         self.set_statusmsg(_("Checking '%s'.") % strformat.limit(url, 40))
-        url_data = checker.get_url_from(url, 0, aggregate)
-        try:
-            add_intern_pattern(url_data, self.config)
-        except UnicodeError:
-            self.set_statusmsg(_("Error, invalid URL `%s'.") %
-                                  strformat.limit(url, 40))
-            return
+        url_data = checker.get_url_from(url, 0, aggregate, extern=(0, 0))
         self.recent.add_document(url)
         aggregate.urlqueue.put(url_data)
         self.aggregate = aggregate
@@ -469,11 +544,18 @@ Version 2 or later.
 
     def set_statusmsg (self, msg):
         """Show given status message."""
-        if len(msg) > 30:
+        self.statusBar.showMessage(msg)
+        if len(msg) > MaxMessageLength:
             self.label_status.setToolTip(msg)
-            msg = msg[:27]+u"..."
+            msg = msg[:MaxMessageLength-3]+u"..."
+        else:
+            self.label_status.setToolTip(u"")
         self.label_status.setText(msg)
 
+    def hover_link (self, link):
+        """Show given link in status bar."""
+        self.statusBar.showMessage(link)
+
     def log_status (self, checked, in_progress, queued, duration):
         """Update number of checked, active and queued URLs."""
         self.label_checked.setText(u"%d" % checked)
@@ -487,3 +569,22 @@ Version 2 or later.
     def internal_error (self, msg):
         """Display internal error message. Triggered by sys.excepthook()."""
         QtGui.QMessageBox.warning(self, _(u"LinkChecker internal error"), msg)
+
+    def handleDragEvent(self, event):
+        """Handle drag enter of move event."""
+        mime = event.mimeData()
+        if not mime.hasUrls():
+            return event.ignore()
+        url = mime.urls()[0]
+        if url.scheme() != 'file':
+            return event.ignore()
+        if not url.path().toLower().endsWith(ProjectExt):
+            return event.ignore()
+        event.accept()
+
+    def handleDropEvent(self, event):
+        """Handle drop event."""
+        mime = event.mimeData()
+        url = mime.urls()[0]
+        filename = unicode(url.toLocalFile())
+        loadproject(self, filename)
