@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2008-2010 Bastian Kleineidam
+# Copyright (C) 2008-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,31 +22,24 @@ class CheckerThread (QtCore.QThread):
     """Separate checker thread."""
 
     def __init__ (self, parent=None):
+        """Reset check variables."""
         super(CheckerThread, self).__init__(parent)
-        self.exiting = False
         self.aggregate = None
-        self.progress = None
 
-    def __del__(self):
-        self.exiting = True
-        self.wait()
-
-    def check (self, aggregate, progress):
+    def check (self, aggregate):
+        """Set check variables and start the thread."""
         self.aggregate = aggregate
-        self.progress = progress
-        self.progress.cancelButton.clicked.connect(self.cancel)
         # setup the thread and call run()
         self.start()
 
     def cancel (self):
-        # stop checking
-        if self.progress is not None:
-            self.progress = None
+        """Reset check variables and set stop flag."""
         if self.aggregate is not None:
-            self.aggregate.wanted_stop = True
+            aggregate = self.aggregate
             self.aggregate = None
+            aggregate.cancel()
 
     def run (self):
-        # start checking
+        """Start checking."""
         assert self.aggregate.config["threads"] > 0
         director.check_urls(self.aggregate)
