@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2009-2010 Bastian Kleineidam
+# Copyright (C) 2009-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,26 +32,39 @@ class GuiLogHandler (Handler, object):
         self.signal.emit(self.format(record))
 
 
-class GuiLogger (Logger):
-    """Delegate log URLs to the UI tree widget."""
+class SignalLogger (Logger):
+    """Use Qt signals for logged URLs and statistics."""
 
     def __init__ (self, **args):
-        super(GuiLogger, self).__init__(**args)
+        """Store signals for URL and statistic data."""
+        super(SignalLogger, self).__init__(**args)
         self.log_url_signal = args["signal"]
         self.log_stats_signal = args["stats"]
 
     def start_fileoutput (self):
+        """Override fileoutput handling of base class."""
         pass
 
     def close_fileoutput (self):
+        """Override fileoutput handling of base class."""
         pass
 
     def log_url (self, url_data):
-        """URL gets logged in the main window."""
+        """Emit URL data which gets logged in the main window."""
         self.log_url_signal.emit(url_data)
 
     def end_output (self):
+        """Emit statistic data which gets logged in the main window."""
         self.log_stats_signal.emit(self.stats)
 
-    def start_output (self):
-        pass
+
+class StatusLogger (object):
+    """GUI status logger, signaling to progress labels."""
+
+    def __init__ (self, signal):
+        """Store given signal object."""
+        self.signal = signal
+
+    def log_status (self, checked, in_progress, queued, duration):
+        """Emit signal with given status information."""
+        self.signal.emit(checked, in_progress, queued, duration)
