@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2001-2011 Bastian Kleineidam
+# Copyright (C) 2001-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,39 +27,60 @@ MAX_TITLELEN = 256
 
 unquote = strformat.unquote
 
-# ripped mainly from HTML::Tagset.pm
+# HTML4/5 link tags
+# ripped mainly from HTML::Tagset.pm with HTML5 added
 LinkTags = {
     'a':        [u'href'],
     'applet':   [u'archive', u'src'],
     'area':     [u'href'],
+    'audio':    [u'src'], # HTML5
     'bgsound':  [u'src'],
     'blockquote': [u'cite'],
     'body':     [u'background'],
+    'button':   [u'formaction'], # HTML5
     'del':      [u'cite'],
     'embed':    [u'pluginspage', u'src'],
     'form':     [u'action'],
     'frame':    [u'src', u'longdesc'],
     'head':     [u'profile'],
+    'html':     [u'manifest'], # HTML5
     'iframe':   [u'src', u'longdesc'],
     'ilayer':   [u'background'],
     'img':      [u'src', u'lowsrc', u'longdesc', u'usemap'],
-    'input':    [u'src', u'usemap'],
+    'input':    [u'src', u'usemap', u'formaction'],
     'ins':      [u'cite'],
     'isindex':  [u'action'],
     'layer':    [u'background', u'src'],
     'link':     [u'href'],
     'meta':     [u'content', u'href'],
-    'object':   [u'classid', u'data', u'archive', u'usemap'],
+    'object':   [u'classid', u'data', u'archive', u'usemap', u'codebase'],
     'q':        [u'cite'],
     'script':   [u'src'],
+    'source':   [u'src'], # HTML5
     'table':    [u'background'],
     'td':       [u'background'],
     'th':       [u'background'],
     'tr':       [u'background'],
+    'track':    [u'src'], # HTML5
+    'video':    [u'src'], # HTML5
     'xmp':      [u'href'],
     None:       [u'style'],
 }
 
+# HTML anchor tags
+AnchorTags = {
+    'a': [u'name'],
+    None: [u'id'],
+}
+
+# WML tags
+WmlTags = {
+    'a':   [u'href'],
+    'go':  [u'href'],
+    'img': [u'src'],
+}
+
+
 # matcher for <meta http-equiv=refresh> tags
 refresh_re = re.compile(ur"(?i)^\d+;\s*url=(?P<url>.+)$")
 _quoted_pat = ur"('[^']+'|\"[^\"]+\"|[^\)\s]+)"
@@ -177,8 +198,9 @@ class LinkFinder (TagFinder):
         if tag == "base" and not self.base_ref:
             self.base_ref = unquote(attrs.get_true("href", u''))
         tagattrs = self.tags.get(tag, [])
+        # add universal tag attributes using tagname None
         tagattrs.extend(self.tags.get(None, []))
-        # eliminate duplicate tag attrs
+        # eliminate duplicate tag attributes
         tagattrs = set(tagattrs)
         # parse URLs in tag (possibly multiple URLs in CSS styles)
         for attr in tagattrs:
@@ -190,7 +212,7 @@ class LinkFinder (TagFinder):
             name = self.get_link_name(tag, attrs, attr)
             # possible codebase
             base = u''
-            if tag in ('applet', 'object'):
+            if tag  == 'applet':
                 base = unquote(attrs.get_true('codebase', u''))
             if not base:
                 base = self.base_ref
@@ -236,6 +258,8 @@ class LinkFinder (TagFinder):
             for mo in css_url_re.finditer(url):
                 u = mo.group("url")
                 urls.append(unquote(u, matching=True))
+        elif attr == u'archive':
+            urls.extend(url.split(u','))
         else:
             urls.append(url)
         if not urls:
