@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,15 @@ class GraphLogger (Logger):
         self.nodes = {}
         self.nodeid = 0
 
+    def log_filter_url(self, url_data, do_print):
+        """Update accounting data and log all valid URLs regardless the
+        do_print flag.
+        """
+        self.stats.log_url(url_data, do_print)
+        # ignore the do_print flag and determine ourselves if we filter the url
+        if url_data.valid:
+            self.log_url(url_data)
+
     def get_node (self, url_data):
         """Return new node data or None if node already exists."""
         if not url_data.url:
@@ -42,7 +51,7 @@ class GraphLogger (Logger):
             "url": url_data.url,
             "parent_url": url_data.parent_url,
             "id": self.nodeid,
-            "label": quote(url_data.title),
+            "label": quote(url_data.title if url_data.title else url_data.name),
             "extern": 1 if url_data.extern else 0,
             "checktime": url_data.checktime,
             "dlsize": url_data.dlsize,
