@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,6 +43,10 @@ class NoneLogger (Logger):
         """
         pass
 
+    def log_url (self, url_data):
+        """Do nothing."""
+        pass
+
     def end_output (self):
         """
         Do nothing.
