@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ def sqlify (s):
     """
     if not s:
         return "NULL"
-    return "'%s'" % s.replace("'", "''")
+    return "'%s'" % s.replace("'", "''").replace(os.linesep, r"\n")
 
 
 def intify (s):
@@ -83,7 +83,7 @@ class SQLLogger (Logger):
         """
         self.writeln(u"insert into %(table)s(urlname,"
               "parentname,baseref,valid,result,warning,info,url,line,col,"
-              "name,checktime,dltime,dlsize,cached) values ("
+              "name,checktime,dltime,dlsize,cached,level) values ("
               "%(base_url)s,"
               "%(url_parent)s,"
               "%(base_ref)s,"
@@ -99,7 +99,8 @@ class SQLLogger (Logger):
               "%(dltime)d,"
               "%(dlsize)d,"
               "%(cached)d,"
-              "%(level)d"
+              "%(level)d,"
+              "%(modified)s"
               ")%(separator)s" %
               {'table': self.dbname,
                'base_url': sqlify(url_data.base_url),
@@ -116,9 +117,10 @@ class SQLLogger (Logger):
                'checktime': url_data.checktime,
                'dltime': url_data.dltime,
                'dlsize': url_data.dlsize,
-               'cached': intify(url_data.cached),
+               'cached': 0,
                'separator': self.separator,
                "level": url_data.level,
+               "modified": sqlify(self.format_modified(url_data.modified)),
               })
         self.flush()
 
