@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,31 +26,6 @@ class TextLogger (Logger):
     """
     A text logger, colorizing the output if possible.
 
-    Every Logger has to implement the following functions:
-
-    C{def start_output (self)}
-    Called once to initialize the Logger. Why do we not use __init__(self)?
-    Because we initialize the start time in start_output and __init__ gets
-    not called at the time the checking starts but when the logger object is
-    created.
-    Another reason is that we might want to create several loggers
-    as a default and then switch to another configured output. So we
-    must not print anything out at __init__ time.
-
-    C{def log_filter_url (self, url_data, do_print)}
-    Called every time an url finished checking. All data we checked is in
-    the UrlData object url_data.
-
-    C{def end_output (self)}
-    Called at the end of checking to close filehandles and such.
-
-    Passing parameters to the constructor:
-
-    C{def __init__ (self, **args)}
-    The args dictionary is filled in Config.py. There you can specify
-    default parameters. Adjust these parameters in the configuration
-    files in the appropriate logger section.
-
     Informal text output format spec:
     Output consists of a set of URL logs separated by one or more
     blank lines.
@@ -80,40 +55,43 @@ class TextLogger (Logger):
         self.colorreset = args.get('colorreset', 'default')
 
     def init_fileoutput (self, args):
+        """Colorize file output if possible."""
         super(TextLogger, self).init_fileoutput(args)
         if self.fd is not None:
             self.fd = ansicolor.Colorizer(self.fd)
 
     def start_fileoutput (self):
         """Needed to make file descriptor color aware."""
+        init_color = self.fd is None
         super(TextLogger, self).start_fileoutput()
-        self.fd = ansicolor.Colorizer(self.fd)
+        if init_color:
+            self.fd = ansicolor.Colorizer(self.fd)
 
     def start_output (self):
-        """
-        Write generic start checking info.
-        """
+        """Write generic start checking info."""
         super(TextLogger, self).start_output()
         if self.has_part('intro'):
             self.write_intro()
         self.flush()
 
     def write_intro (self):
+        """Log introduction text."""
         self.writeln(configuration.AppInfo)
         self.writeln(configuration.Freeware)
         self.writeln(_("Get the newest version at %(url)s") %
                      {'url': configuration.Url})
         self.writeln(_("Write comments and bugs to %(url)s") %
                      {'url': configuration.SupportUrl})
+        self.writeln(_("Support this project at %(url)s") %
+                     {'url': configuration.DonateUrl})
         self.check_date()
         self.writeln()
         self.writeln(_("Start checking at %s") %
                      strformat.strtime(self.starttime))
 
     def log_url (self, url_data):
-        """
-        Write url checking info.
-        """
+        """Write url checking info."""
+        self.writeln()
         if self.has_part('url'):
             self.write_url(url_data)
         if url_data.name and self.has_part('name'):
@@ -132,6 +110,8 @@ class TextLogger (Logger):
             self.write_dlsize(url_data)
         if url_data.info and self.has_part('info'):
             self.write_info(url_data)
+        if url_data.modified and self.has_part('modified'):
+            self.write_modified(url_data)
         if url_data.warnings and self.has_part('warning'):
             self.write_warning(url_data)
         if self.has_part('result'):
@@ -148,21 +128,15 @@ class TextLogger (Logger):
         """Write url_data.base_url."""
         self.write(self.part('url') + self.spaces('url'))
         txt = strformat.strline(url_data.base_url)
-        if url_data.cached:
-            txt += _(" (cached)")
         self.writeln(txt, color=self.colorurl)
 
     def write_name (self, url_data):
-        """
-        Write url_data.name.
-        """
+        """Write url_data.name."""
         self.write(self.part("name") + self.spaces("name"))
         self.writeln(strformat.strline(url_data.name), color=self.colorname)
 
     def write_parent (self, url_data):
-        """
-        Write url_data.parent_url.
-        """
+        """Write url_data.parent_url."""
         self.write(self.part('parenturl') + self.spaces("parenturl"))
         txt = url_data.parent_url
         txt += _(", line %d") % url_data.line
@@ -170,61 +144,51 @@ class TextLogger (Logger):
         self.writeln(txt, color=self.colorparent)
 
     def write_base (self, url_data):
-        """
-        Write url_data.base_ref.
-        """
+        """Write url_data.base_ref."""
         self.write(self.part("base") + self.spaces("base"))
         self.writeln(url_data.base_ref, color=self.colorbase)
 
     def write_real (self, url_data):
-        """
-        Write url_data.url.
-        """
+        """Write url_data.url."""
         self.write(self.part("realurl") + self.spaces("realurl"))
         self.writeln(unicode(url_data.url), color=self.colorreal)
 
     def write_dltime (self, url_data):
-        """
-        Write url_data.dltime.
-        """
+        """Write url_data.dltime."""
         self.write(self.part("dltime") + self.spaces("dltime"))
         self.writeln(_("%.3f seconds") % url_data.dltime,
                      color=self.colordltime)
 
     def write_dlsize (self, url_data):
-        """
-        Write url_data.dlsize.
-        """
+        """Write url_data.dlsize."""
         self.write(self.part("dlsize") + self.spaces("dlsize"))
         self.writeln(strformat.strsize(url_data.dlsize),
                      color=self.colordlsize)
 
     def write_checktime (self, url_data):
-        """
-        Write url_data.checktime.
-        """
+        """Write url_data.checktime."""
         self.write(self.part("checktime") + self.spaces("checktime"))
         self.writeln(_("%.3f seconds") % url_data.checktime,
                      color=self.colordltime)
 
     def write_info (self, url_data):
-        """
-        Write url_data.info.
-        """
+        """Write url_data.info."""
         self.write(self.part("info") + self.spaces("info"))
         self.writeln(self.wrap(url_data.info, 65), color=self.colorinfo)
 
+    def write_modified(self, url_data):
+        """Write url_data.modified."""
+        self.write(self.part("modified") + self.spaces("modified"))
+        self.writeln(self.format_modified(url_data.modified))
+
     def write_warning (self, url_data):
-        """
-        Write url_data.warning.
-        """
+        """Write url_data.warning."""
         self.write(self.part("warning") + self.spaces("warning"))
-        self.writeln(self.wrap(url_data.warnings, 65), color=self.colorwarning)
+        warning_msgs = [u"[%s] %s" % x for x in url_data.warnings]
+        self.writeln(self.wrap(warning_msgs, 65), color=self.colorwarning)
 
     def write_result (self, url_data):
-        """
-        Write url_data.result.
-        """
+        """Write url_data.result."""
         self.write(self.part("result") + self.spaces("result"))
         if url_data.valid:
             color = self.colorvalid
@@ -237,23 +201,38 @@ class TextLogger (Logger):
         self.writeln()
 
     def write_outro (self):
+        """Write end of checking message."""
         self.writeln()
         self.write(_("That's it.") + " ")
         self.write(_n("%d link checked.", "%d links checked.",
                       self.stats.number) % self.stats.number)
         self.write(u" ")
-        self.write(_n("%d warning found", "%d warnings found",
-             self.stats.warnings_printed) % self.stats.warnings_printed)
+        warning_text = _n("%d warning found", "%d warnings found",
+             self.stats.warnings_printed) % self.stats.warnings_printed
+        if self.stats.warnings_printed:
+            warning_color = self.colorwarning
+        else:
+            warning_color = self.colorinfo
+        self.write(warning_text, color=warning_color)
         if self.stats.warnings != self.stats.warnings_printed:
             self.write(_(" (%d ignored or duplicates not printed)") %
                 (self.stats.warnings - self.stats.warnings_printed))
         self.write(u". ")
-        self.write(_n("%d error found", "%d errors found",
-             self.stats.errors_printed) % self.stats.errors_printed)
+        error_text = _n("%d error found", "%d errors found",
+             self.stats.errors_printed) % self.stats.errors_printed
+        if self.stats.errors_printed:
+            error_color = self.colorinvalid
+        else:
+            error_color = self.colorvalid
+        self.write(error_text, color=error_color)
         if self.stats.errors != self.stats.errors_printed:
             self.write(_(" (%d duplicates not printed)") %
                 (self.stats.errors - self.stats.errors_printed))
         self.writeln(u".")
+        num = self.stats.internal_errors
+        if num:
+            self.writeln(_n("There was %(num)d internal error.",
+                "There were %(num)d internal errors.", num) % {"num": num})
         self.stoptime = time.time()
         duration = self.stoptime - self.starttime
         self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
@@ -261,8 +240,13 @@ class TextLogger (Logger):
               "duration": strformat.strduration_long(duration)})
 
     def write_stats (self):
+        """Write check statistic info."""
         self.writeln()
         self.writeln(_("Statistics:"))
+        if self.stats.downloaded_bytes > 0:
+            self.writeln(_("Downloaded: %s") % strformat.strsize(self.stats.downloaded_bytes))
+        hitsmisses = strformat.str_cache_stats(*self.stats.robots_txt_stats)
+        self.writeln(_("Robots.txt cache: %s") % hitsmisses)
         if len(self.stats.domains) > 1:
             self.writeln(_("Number of domains: %d") % len(self.stats.domains))
         if self.stats.number > 0:
@@ -275,12 +259,10 @@ class TextLogger (Logger):
                          max=self.stats.max_url_length,
                          avg=self.stats.avg_url_length))
         else:
-            self.writeln(_("No statistics available since zero URLs were checked."))
+            self.writeln(_("No statistics available since no URLs were checked."))
 
     def end_output (self):
-        """
-        Write end of output info, and flush all output buffers.
-        """
+        """Write end of output info, and flush all output buffers."""
         if self.has_part('stats'):
             self.write_stats()
         if self.has_part('outro'):
